import java.util.Properties;

import javax.transaction.TransactionManager;

import org.hibernate.search.annotations.Field;
import org.hibernate.search.annotations.Indexed;
import org.hibernate.search.annotations.ProvidedId;
import org.infinispan.Cache;
import org.infinispan.manager.CacheManager;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.query.CacheQuery;
import org.infinispan.query.QueryFactory;
import org.infinispan.query.backend.QueryHelper;
import org.infinispan.transaction.lookup.JBossStandaloneJTAManagerLookup;
import org.infinispan.transaction.lookup.TransactionManagerLookup;

public class QueryTest
{
    /**
     * 
     */
    protected CacheManager m_cacheManager;
    
    /**
     * 
     */
    private QueryHelper m_queryHelper;
    
    /**
     * 
     */
    private Cache<String, Session> m_cache;

    /**
     * 
     */
    private TransactionManager m_transactionManager;

    /**
     * 
     * @param args
     */
    public static final void main(String[] args)
    {
        //
        QueryTest queryTest = new QueryTest();
        
        //
        queryTest.initialize("datagrid-infinispan.xml");
        
        //
        queryTest.run();
    }

    /**
     * 
     * @param config
     */
    private void initialize(String config)
    {
        try
        {
            // Create the cache manager
            m_cacheManager = new DefaultCacheManager(config);

            // Initialize the transaction manager
            TransactionManagerLookup tml = new JBossStandaloneJTAManagerLookup();
            m_transactionManager = tml.getTransactionManager();
                        
            // Create the cache
            m_cache = m_cacheManager.getCache("Ready");
            
            // Make the hibernate cache an in memory cache
            Properties properties = new Properties();
            properties.put("hibernate.search.default.directory_provider", "org.hibernate.search.store.RAMDirectoryProvider");
            
            // Initialze the query helper.
            m_queryHelper = new QueryHelper(m_cache, properties, Session.class);
            
            // Initialize the cache
            m_transactionManager.begin();
            for (int i=0; i<100; i++)
            {
                m_cache.put(String.valueOf(i), new Session(String.valueOf(i)));
            }
            m_transactionManager.commit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }        
    }

    /**
    * 
    */
    public void run()
    {
        try
        {
            // Verify querying works
            QueryFactory queryFactory = new QueryFactory(m_cache, m_queryHelper);
            CacheQuery cacheQuery = queryFactory.getBasicQuery("", "Id:2?");
            System.out.println("Hits: "+cacheQuery.getResultSize());
            
            // Remove something that exists
            m_transactionManager.begin();
            m_cache.remove("50");
            m_transactionManager.commit();
            
            // Remove something that doesn't exist with a transaction
            // This also fails without using a transaction
            m_transactionManager.begin();
            m_cache.remove("200");            
            m_transactionManager.commit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    @ProvidedId
    @Indexed(index="SessionIndex")
    public class Session
    {
        private String m_id;
        
        public Session(String id)
        {
            m_id = id;
        }
        
        @Field(name="Id")
        public String getId() { return m_id; }
    }
}
